@@ -5,8 +5,8 @@
 # Author          : Johan Vromans
 # Created On      : Fri Jan 15 19:15:00 2016
 # Last Modified By: Johan Vromans
-# Last Modified On: Mon Oct 10 15:02:43 2016
-# Update Count    : 1416
+# Last Modified On: Tue Nov 15 16:15:56 2016
+# Update Count    : 1457
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -22,7 +22,7 @@ package Data::iRealPro::Output::Imager;
 
 use parent qw( Data::iRealPro::Output::Base );
 
-our $VERSION = "0.10";
+our $VERSION = "0.12";
 
 use Data::Dumper;
 use Text::CSV_XS;
@@ -35,20 +35,28 @@ sub new {
 
     my $self = bless( { variant => "irealpro" }, $pkg );
 
-    $self->{fontdir} = $ENV{FONTDIR};
+    # Get the resource dir.
+    $self->{resdir} = $ENV{RESDIR};
     if ( $App::Packager::PACKAGED ) {
-	$self->{resdir} = App::Packager::GetResourcePath();
+	$self->{resdir} ||= App::Packager::GetResourcePath();
     }
     else {
-	$self->{resdir} = "$FindBin::Bin/../res";
+	$self->{resdir} ||= "$FindBin::Bin/../res";
     }
-    $self->{fontdir} ||= $self->{resdir} . "/fonts";
+    $self->{resdir} .= "/";
+    $self->{resdir} =~ s;/+$;/;;
+
+    # Fonts resource dir.
+    $self->{fontdir} = $ENV{FONTDIR} || $self->{resdir} . "/fonts";
     $self->{fontdir} .= "/";
     $self->{fontdir} =~ s;/+$;/;;
 
     if ( $options->{npp} ) {
 	die( "Unsupported output type for NPP. Please select PNG or JPG.\n")
 	  unless $options->{output} =~ /\.(jpg|png)$/i;
+	if ( $options->{npp} =~ s/_strict// ) {
+	    $options->{npp_strict} = 1;
+	}
 	if ( $options->{npp} =~ s/-$// ) {
 	    $options->{npp_minor} = '';
 	}
@@ -56,13 +64,21 @@ sub new {
 	    $options->{npp_minor} = 'm';
 	}
 	$options->{npp} = 'straight' unless $options->{npp} eq 'hand';
-	unless ( -s $self->{resdir} . "/prefab/" . $options->{npp} . "/root_c.png" ) {
+	$self->{prefab} = $self->{resdir} . "/drawable-nodpi-v4/";
+	# Quality_H is the 'missing symbol' symbol. It better be there.
+	if ( -s $self->{prefab} . "quality_h" .
+	        ( $options->{npp} eq 'hand' ? "_hand" : "" ) .
+	         ".png" ) {
+	}
+	else {
 	    die( "NPP Image generation not available" );
 	}
+	warn("Using NPP images from ", $self->{prefab}, "\n")
+	  if $options->{verbose};
     }
 
     for ( qw( trace debug verbose output variant transpose toc crop
-	      npp npp_minor
+	      npp npp_minor npp_strict
 	   ) ) {
 	$self->{$_} = $options->{$_} if exists $options->{$_};
     }
@@ -509,12 +525,6 @@ sub make_image {
 	for ( $cell->text ) {
 	    next unless $_;
 	    my ( $disp, $t ) = @$_;
-	    # Displacement is 0 .. 74, in steps of 3.
-	    if ($self->{npp} ) {
-		$self->textl( $x-2, $y + $dy - 27 - ($dy / 74) * $disp, $t,
-			      74, $textfont, $red );
-		next;
-	    }
 
 	    if ( FONTSX && $self->{pdf} ) {
 		for ( split( //, $t ) ) {
@@ -522,15 +532,25 @@ sub make_image {
 		    warn( sprintf( "Missing glyph U+%04X\n", ord($_) ) );
 		}
 	    }
+
+	    my $hack;
 	    # Sometimes, THAI PAIYANNOI (U+2e7) is abused as
 	    # MUSICAL SYMBOL EIGHTH REST (u+1d13e).
-	    $t =~ s/\x{e2f}/\x{1d13e}/g;
+	    $t =~ s/\x{e2f}/\x{1d13e}/g && $hack++;
 	    # Likewise CYRILLIC SMALL LETTER GHE WITH UPTURN (U+491)
 	    # -> MUSICAL SYMBOL QUARTER REST (U+1D13D)
-	    $t =~ s/\x{491}/\x{1d13d}/g;
+	    $t =~ s/\x{491}/\x{1d13d}/g && $hack++;
 	    # Likewise BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE (U+2555)
 	    # -> MUSICAL SYMBOL SIXTEENTH REST (U+1D13F)
-	    $t =~ s/\x{2555}/\x{1d13f}/g;
+	    $t =~ s/\x{2555}/\x{1d13f}/g && $hack++;
+
+	    # Displacement is 0 .. 74, in steps of 3.
+	    if ($self->{npp} ) {
+		$self->textl( $x-2, $y + $dy - 27 - ($dy / 74) * $disp, $t,
+			      $hack ? 60 : 74, $textfont, $red );
+		next;
+	    }
+
 	    $self->textl( $x+0.15*$musicsize,
 			  $y+0.55*$musicsize-($disp/(45/$musicsize)),
 			  $t, 0.55*$musicsize, $textfont, $red );
@@ -648,6 +668,7 @@ sub make_image {
 	}
 
 	if ( $cell->flags && $cell->flags & 0x01 ) { # invisible END
+	    next;				     # suppress.
 	    my $disp = 0;
 	    if ( $self->{npp} ) {
 		$self->textl( $x-2, $y + $dy - 27 - ($dy / 74) * $disp,
@@ -1228,16 +1249,30 @@ sub getimg {
     my ( $self, $img ) = @_;
     return $npp_imgcache{$img} if $npp_imgcache{$img};
 
-    my $if = $self->{resdir} . "/prefab/" . $self->{npp} . "/$img.png";
+    my $if = $self->{prefab} . $img . ".png";
+
+    if ( $self->{npp} eq "hand" ) {
+	# iRealPro uses some non-hand symbols even though nice alternatives exist.
+	unless ( $self->{npp_strict} && $img =~ /rehearsal_mark|root_nc|root_xx/ ) {
+	    $if =~ s/\.png/_hand.png/;
+	}
+	unless ( -s $if ) {
+	    # Fallback to non-hand symbols.
+	    $if =~ s/_hand\././;
+	}
+    }
+
     my $red = 0;
     unless ( -s $if ) {
 	$red = 1;
 	warn("Substituting <notfound> for \"$img\"\n");
-	$if = $self->{resdir} . "/prefab/" . $self->{npp} . "/quality_h.png";
+	$if = $self->{prefab} . "quality_h.png";
+	$if =~ s/\.png/_hand.png/ if $self->{npp} eq "hand";
     }
     $npp_imgcache{$img} = Imager->new( file => $if )
       or die( Imager->errstr );
 
+    # Some symbols are rendered in red.
     $red ||=  $img =~ /^ (?:
 			   .*rehearsal_mark_. |
 			   repeat_barline_(?:open|close) |
@@ -1245,6 +1280,12 @@ sub getimg {
 			   ending_.* |
 			   coda | fermata | segno
 		       ) $/x;
+
+    # The barline images are white instead of black. Reverse.
+    $npp_imgcache{$img}->map( all => [ reverse( 0..255) ], alpha => [] )
+      if $img =~ /(single|double)_barline/;
+
+    # Colourize if necessary.
     $npp_imgcache{$img}->map( red => [ reverse( 0..255) ] ) if $red;
 
     return $npp_imgcache{$img};
